/*
	zval* param = NULL;
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2010 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author:                                                              |
  +----------------------------------------------------------------------+
*/

/* $Id: header 297205 2010-03-30 21:09:07Z johannes $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_reports.h"

#include "reports.h"

/* If you declare any globals in php_reports.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(php_reports)
*/

/* True global resources - no need for thread safety here */
static int le_php_reports;

/* {{{ php_reports_functions[]
 *
 * Every user visible function must have an entry in php_reports_functions[].
 */
const zend_function_entry php_reports_functions[] = {
	PHP_FE(fr_version, NULL)
	PHP_FE(fr_set_log_level, NULL)
	PHP_FE(fr_set_log_flag, NULL)
	PHP_FE(fr_set_defaults, NULL)
	PHP_FE(fr_renders, NULL)
	PHP_FE(fr_render, NULL)
	{NULL, NULL, NULL}	/* Must be the last line in php_reports_functions[] */
};
/* }}} */

/* {{{ php_reports_module_entry
 */
zend_module_entry php_reports_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"php_reports",
	php_reports_functions,
	PHP_MINIT(php_reports),
	PHP_MSHUTDOWN(php_reports),
	PHP_RINIT(php_reports),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(php_reports),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(php_reports),
#if ZEND_MODULE_API_NO >= 20010901
	PHP_REPORTS_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_PHP_REPORTS
ZEND_GET_MODULE(php_reports)
#endif

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("php_reports.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_php_reports_globals, php_reports_globals)
    STD_PHP_INI_ENTRY("php_reports.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_php_reports_globals, php_reports_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_reports_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_reports_init_globals(zend_php_reports_globals *php_reports_globals)
{
	php_reports_globals->global_value = 0;
	php_reports_globals->global_string = NULL;
}
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(php_reports)
{
	rp_init();
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(php_reports)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(php_reports)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(php_reports)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(php_reports)
{
	php_printf("Field Reports PHP bridge\n");
	php_info_print_table_start();
	php_info_print_table_row(2, "Version", PHP_REPORTS_VERSION);
	php_info_print_table_row(2, "Authors", "Field Works, LLC. 'info@field-works.co.jp'\n");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

static int is_hash(zval* obj)
{
#if PHP_MAJOR_VERSION >= 7
	HashTable* hash = NULL;
	zend_ulong num_key;
	zend_string* key;
	zval *val;
	int result = 0;

	hash = Z_ARRVAL_P(obj);
	ZEND_HASH_FOREACH_KEY_VAL(hash, num_key, key, val) {
		if (key) {
			result = 1;
			break;
		}
	}
	ZEND_HASH_FOREACH_END();
	return result;
#else
	HashTable* hash = NULL;
	zval** entry = NULL;
	char* key = NULL;
	unsigned long int idx = 0;
	int type;

	hash = Z_ARRVAL_P(obj);
	zend_hash_internal_pointer_reset(hash);
	while (zend_hash_get_current_data(hash, (void**)&entry) == SUCCESS) {
		type = zend_hash_get_current_key(hash, &key, &idx, 0);
		if (type == HASH_KEY_IS_STRING) {
			return 1;
		}
		zend_hash_move_forward(hash);
	}
	return 0;
#endif
}

static int php_type(zval* obj)
{
	switch (Z_TYPE_P(obj)) {
	case IS_NULL:
		return 0 /*RP_TYPE_NULL*/;
	case IS_LONG:
		return RP_TYPE_INT;
	case IS_DOUBLE:
		return RP_TYPE_FLOAT;
#if PHP_MAJOR_VERSION >= 7
	case IS_TRUE:
	case IS_FALSE:
#else
    case IS_BOOL:
#endif
		return RP_TYPE_BOOL;
	case IS_ARRAY:
		if (is_hash(obj)) {
			return RP_TYPE_OBJECT;
		} else {
			return RP_TYPE_ARRAY;
		}
	case IS_STRING:
		return RP_TYPE_STRING;
	default:
		return RP_TYPE_OTHER;
	}
}

static void php_release(zval* opt)
{
}

static int php_val_bool(zval* obj, zval** opt)
{
#if PHP_MAJOR_VERSION >= 7
	return Z_TYPE_P(obj) == IS_TRUE;
#else
	return Z_BVAL_P(obj);
#endif
}

static zend_long php_val_int(zval* obj, zval** opt)
{
    return Z_LVAL_P(obj);
}

static double php_val_float(zval* obj, zval** opt)
{
    return Z_DVAL_P(obj);
}

static char* php_val_string(zval* obj, zval** opt)
{
    return Z_STRVAL_P(obj);
}

static HashTable* php_array_head(zval* obj, zval** opt)
{
	HashTable* hash = Z_ARRVAL_P(obj);
	zend_hash_internal_pointer_reset(hash);
	return hash;
}

int php_array_next(zval* obj, HashTable** hash, zval** entry, zval** opt)
{
#if PHP_MAJOR_VERSION >= 7
	*entry = zend_hash_get_current_data(*hash);
	if (*entry) {
		zend_hash_move_forward(*hash);
		return 1;
	}
	return 0;
#else
	zval** zentry = NULL;
	if (zend_hash_get_current_data(*hash, (void**)&zentry) == SUCCESS) {
		*entry = *zentry;
		zend_hash_move_forward(*hash);
		return 1;
	}
	return 0;
#endif
}

static HashTable* php_object_head(zval* obj, zval** opt)
{
	HashTable* hash = Z_ARRVAL_P(obj);
	zend_hash_internal_pointer_reset(hash);
	return hash;
}

static int php_object_next(zval* obj, HashTable** hash, char** key, zval** entry, zval** opt)
{
#if PHP_MAJOR_VERSION >= 7
	*entry = zend_hash_get_current_data(*hash);
	if (*entry) {
		zval zkey;
		zend_hash_get_current_key_zval(*hash, &zkey);
		*key = Z_STRVAL(zkey);
		zend_hash_move_forward(*hash);
		return 1;
	}
	return 0;
#else
	zval** zentry = NULL;
	if (zend_hash_get_current_data(*hash, (void**)&zentry) == SUCCESS) {
		unsigned long int idx = 0;
		int type = zend_hash_get_current_key(*hash, key, &idx, 0);
		if (type != HASH_KEY_IS_STRING) {
			static char buf[20];
			snprintf(buf, sizeof(buf), "%d", idx);
			*key = buf;
		}
		*entry = *zentry;
		zend_hash_move_forward(*hash);
		return 1;
	}
	return 0;
#endif
}

rp_callback cb = {
    (t_fn_type)php_type,
    (t_fn_val_bool)php_val_bool,
    (t_fn_val_int)php_val_int,
    (t_fn_val_float)php_val_float,
    (t_fn_val_string)php_val_string,
    (t_fn_array_head)php_array_head,
    (t_fn_array_next)php_array_next,
    (t_fn_object_head)php_object_head,
    (t_fn_object_next)php_object_next,
    (t_fn_release)php_release,
};

#define SET_RETURN_STRING(v) set_return_string(return_value, v)

static void set_return_string(zval* return_value, caml_value* v)
{
    int len = rp_get_string_val(v, NULL);
    char* buf = malloc(len + 1);
    if (buf) {
        rp_get_string_val(v, buf);
        buf[len] = '\0';
#if PHP_MAJOR_VERSION >= 7
		ZVAL_STRINGL(return_value, buf, len);
#else
		ZVAL_STRINGL(return_value, buf, len, 1);
#endif
        free(buf);
    }
}

static void set_error_msg(caml_value* exn)
{
	char msg[RP_MAX_ERR_MSG_LEN];
	int err;
	const char* tag = rp_get_error(exn, &err, msg);
	TSRMLS_FETCH();
	php_error_docref(NULL TSRMLS_CC, E_ERROR, "%s: %s", tag, msg);
}

/* {{{ proto string fr_version()
  Get the version number. */
PHP_FUNCTION(fr_version)
{
	caml_value* caml_result = NULL;

	if (ZEND_NUM_ARGS() > 0) {
		zend_wrong_param_count(TSRMLS_C);
		goto exit;
	}
	caml_result = rp_version();
	if (rp_is_error(caml_result)) {
		set_error_msg(caml_result);
		goto exit;
	}
	SET_RETURN_STRING(caml_result);

exit:
	if (caml_result) rp_free(caml_result);
}
/* }}} fr_version */


/* {{{ proto void fr_set_log_level(int n)
  Set log level. */
PHP_FUNCTION(fr_set_log_level)
{
	long n = 0;
	caml_value* caml_result = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &n) == FAILURE) {
		goto exit;
	}
	caml_result = rp_set_log_level(n);
	if (rp_is_error(caml_result)) {
		set_error_msg(caml_result);
		goto exit;
	}

exit:
	if (caml_result) rp_free(caml_result);
}
/* }}} fr_set_log_level */


/* {{{ proto void fr_set_log_flag(int n)
  Set log flag. */
PHP_FUNCTION(fr_set_log_flag)
{
	long f = 0;
	caml_value* caml_result = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &f) == FAILURE) {
		goto exit;
	}
	caml_result = rp_set_log_flag(f);
	if (rp_is_error(caml_result)) {
		set_error_msg(caml_result);
		goto exit;
	}

exit:
	if (caml_result) rp_free(caml_result);
}
/* }}} fr_set_log_flag */


/* {{{ proto void fr_set_defaults(mixed param)
  Set default parameters. */
PHP_FUNCTION(fr_set_defaults)
{
	zval* param = NULL;
	HashTable* hash = NULL;
	zval** entry = NULL;
	char* key = NULL;
	unsigned long int idx = 0;
	caml_value* caml_param = NULL;
	caml_value* caml_result = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z/", &param) == FAILURE) {
		goto exit;
	}
    caml_param = rp_encode_json(&cb, param);
    if (rp_is_error(caml_param)) {
        set_error_msg(caml_param);
        goto exit;
    }
    caml_result = rp_set_defaults(caml_param);
    if (rp_is_error(caml_result)) {
        set_error_msg(caml_result);
        goto exit;
    }

exit:
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}
/* }}} fr_set_defaults */


/* {{{ proto string fr_renders(mixed param)
  Render PDF string. */
PHP_FUNCTION(fr_renders)
{
	zval* param = NULL;
	caml_value* caml_param = NULL;
	caml_value* caml_result = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z/", &param) == FAILURE) {
		goto exit;
	}
    caml_param = rp_encode_json(&cb, param);
    if (rp_is_error(caml_param)) {
        set_error_msg(caml_param);
        goto exit;
    }
    caml_result = rp_pdf_of_json(caml_param);
    if (rp_is_error(caml_result)) {
        set_error_msg(caml_result);
        goto exit;
    }
	SET_RETURN_STRING(caml_result);

exit:
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}
/* }}} fr_renders */


/* {{{ proto void fr_render(mixed param, string filename)
  Render PDF and write to file. */
PHP_FUNCTION(fr_render)
{
	zval* param = NULL;
	char* filename = NULL;
#if PHP_MAJOR_VERSION >= 7
	size_t filename_len = 0;
#else
	int filename_len = 0;
#endif
	caml_value* caml_param = NULL;
	caml_value* caml_result = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z/s",
				&param, &filename, &filename_len) == FAILURE) {
		goto exit;
	}
    caml_param = rp_encode_json(&cb, param);
    if (rp_is_error(caml_param)) {
        set_error_msg(caml_param);
        goto exit;
    }
    caml_result = rp_write_pdf_from_json(caml_param, filename);
    if (rp_is_error(caml_result)) {
        set_error_msg(caml_result);
        goto exit;
    }

exit:
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}
/* }}} fr_render */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */

